package org.intellij.ipp.fqnnames;

import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiJavaCodeReferenceElement;
import com.intellij.psi.PsiJavaFile;
import com.intellij.psi.util.PsiTreeUtil;
import com.intellij.util.IncorrectOperationException;
import org.intellij.ipp.util.FQNUtil;
import org.jetbrains.annotations.NotNull;

/**
 * Copyright 2006 Cojocar Dan
 * <p/>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p/>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p/>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * <b>Replace all qualified name references with import</b> is a new intention that will
 *  replace all occurrences of qualified Java class with their short version. This intention is working like the version
 *  above except that will change all the elements that are the same with the element selected.
 * 
 * User: dan
 * Date: Oct 24, 2006
 * Time: 2:52:02 PM
 * $Id$
 */
public class FQNAllIntention extends FQNIntention {

    /**
     * Intention mechanism @see com.siyeh.ipp.base.Intention#processIntention
     * @param element the element that we will inspect
     * @throws IncorrectOperationException
     */
    public void processIntention(@NotNull PsiElement element)
            throws IncorrectOperationException {
        final PsiJavaFile javaFile = PsiTreeUtil.getParentOfType(element, PsiJavaFile.class);
        if (javaFile == null) {
            return;
        }
        FQNIntention intention = new FQNIntention();
        PsiJavaCodeReferenceElement referenceElement = FQNUtil.getReferencedName(element);
        if (referenceElement != null) {
            String fqnName = referenceElement.getQualifiedName();
            processAll(intention, javaFile, fqnName);
        }
    }

    /**
     * This will iterate all the elements in the selected file and will try to apply the @link FQNIntention
     * @param intention the @link FQNIntention
     * @param element the element that we inspect
     * @param fqnName the full name of the element that we selected
     * @throws IncorrectOperationException
     */
    private void processAll(FQNIntention intention, PsiElement element, String fqnName) throws IncorrectOperationException {
        if (intention.getElementPredicate().satisfiedBy(element)) {
            PsiJavaCodeReferenceElement referenceElement = FQNUtil.getReferencedName(element);
            if (referenceElement != null) {
                String elemnetFqnName = referenceElement.getQualifiedName();
                if (elemnetFqnName.equals(fqnName))
                    intention.processIntention(element);
            }
        }
        for (PsiElement el : element.getChildren())
            processAll(intention, el, fqnName);
    }

}
