package org.intellij.ipp.fqnnames;

import com.intellij.openapi.project.Project;
import com.intellij.psi.*;
import com.intellij.psi.codeStyle.CodeStyleSettings;
import com.intellij.psi.codeStyle.CodeStyleSettingsManager;
import com.intellij.psi.util.PsiTreeUtil;
import com.siyeh.ipp.base.PsiElementPredicate;
import org.intellij.ipp.util.FQNUtil;

/**
 * Copyright 2006 Cojocar Dan
 * <p/>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p/>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p/>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * User: dan
 * Date: Oct 24, 2006
 * Time: 2:55:31 PM
 * $Id$
 */
class FQNPredicate implements PsiElementPredicate {
    public boolean satisfiedBy(PsiElement element) {
        if (element instanceof PsiIdentifier){
            while (element!=null&& element.getParent()!=null&&
                    !(element.getParent() instanceof PsiJavaCodeReferenceElement))
                element=element.getParent();
        }
        if (!(element instanceof PsiJavaCodeReferenceElement)) {
            return false;
        }
        PsiJavaCodeReferenceElement referenceElement=FQNUtil.getReferencedName(element);
        if (!referenceElement.isQualified()) {
            return false;
        }

        if (!(referenceElement.resolve() instanceof PsiClass))
            return false;

        String qualifiedName=referenceElement.getCanonicalText();
        final PsiJavaFile javaFile=PsiTreeUtil.getParentOfType(referenceElement, PsiJavaFile.class);
        if (javaFile == null)
            return false;
        if (!FQNUtil.nameCanBeImported(qualifiedName, javaFile))
            return false;

        if (PsiTreeUtil.getParentOfType(element, PsiImportStatementBase.class,PsiPackageStatement.class, PsiCodeFragment.class) != null) {
            return false;
        }
        final PsiElement qualifier = referenceElement.getQualifier();
        if (qualifier instanceof PsiJavaCodeReferenceElement) {
            final PsiJavaCodeReferenceElement qualfierReferenceElement =
                    (PsiJavaCodeReferenceElement) qualifier;
            final PsiElement resolved = qualfierReferenceElement.resolve();
            if (resolved instanceof PsiPackage) {
                return true;
            }
            if (resolved instanceof PsiClass) {
                final Project project = element.getProject();
                final CodeStyleSettings codeStyleSettings =
                        CodeStyleSettingsManager.getSettings(project);
                return codeStyleSettings.INSERT_INNER_CLASS_IMPORTS;
            }
        }
        return false;
    }
}
